clear;close all;clc;
tic
rand('state',sum(100*clock));   %reset the random number generator

N=200;     % Number of points in each round-trip time                   
Nrt=12000; % The number of cycles in the OEO               
Nmax=10000; % The number of iterations to reach a steady state is (Nrt-Nmax)=2000                           
T=2e-6; %round trip time (T is like tau=L/c in the paper )                 
R=50;
RFampCorr=1; %Correction factor to RF amplifier gain
P0=30e-3; %Input optical power
VpiAC=3.14;  %V_pi,AC                      
VpiDC=3.14; %V_pi_DC=3.14 V
Vb=3.14; %Bias voltage Vb              
eta=1;  %The parameter etha as in the paper                        
rhoDC=0.8; %responsivity of the photodetector at 10 GHz (A/W)           
rho=0.8;  %responsivity of the photodetector at DC (A/W)     
Ga=7.5;   %amplifier voltage gain             
attenRF=10^(0); %RF attenuation (default value = 1)
FilterLoss=10^(0); %Filter loss
Gwidth=20e6;  %The bandwidth of the filter (like Gamma in the paper)              
CouplerLoss=10^(0);
rho_N=10^(-20);
Gs=1.5;
alpha=(2*VpiDC*Gs)/(eta*pi*P0*rho*R*Ga); %insertion loss 

p1=0;p2=0;p3=0;
delta=1e-1;

display(['Generating figure 8. Please wait.']);

while p1<1 || p2<1 || p3<1
    [phase_noise,phase_noise_frq,v0_v,t_v,freq_v,phase_noise_RF,phi_v]=SingleOEO_main(N,Nrt,Nmax,T,alpha,RFampCorr,P0,VpiAC,VpiDC,Vb,eta,rhoDC,rho,Ga,...
        attenRF,FilterLoss,Gwidth,CouplerLoss,rho_N);

    f_axis=freq_v.*T;
    [Y_max I]=max(phase_noise_RF);
    [Y_max J]=max(real(v0_v));
    [Y_min J]=min(real(v0_v));
    Y_osc=(Y_max-Y_min)/Y_min;
    
    f_cent=f_axis(I);
    if (abs(f_cent)<delta) &&  Y_min>1.7 && Y_osc<0.1 && p1<1
        p1=1;
        phase_noise_RF1=phase_noise_RF;
        v0_v1=v0_v;
        display(['Generated figure 8(a). Please wait.']);
%         figure(1);
%         subplot(3,2,1)
%         plot(freq_v.*T,phase_noise_RF,'-b','LineWidth',1.5);
%         box off;
%         set(gca,'fontname','Times New Roman','fontsize',12);
%         set(gca,'Xlim',[-2.2 2.2],'Xtick',[-2,0,2],'XtickLabel',{'-2';'0';'2'});
%         set(gca,'Ylim',[-0.1 1.1],'Ytick',[0,1],'YtickLabel',{'0';'1'});
%         xlabel('$f\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
%         ylabel('$S_{RF}^{M\tau } (f)\delta f''$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
% 
%         subplot(3,2,2)
%         plot(t_v./T,real(v0_v).*pi./VpiDC,'-b','LineWidth',1.5);
%         box off;
%         set(gca,'fontname','Times New Roman','fontsize',12);
%         set(gca,'Xlim',[0 5],'Xtick',[0,5],'XtickLabel',{'0';'5'});
%         set(gca,'Ylim',[-2.5 2.5],'Ytick',[-2,2],'YtickLabel',{'-2';'2'});
%         xlabel('$T/\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
%         ylabel('${\mathop{\rm Re}\nolimits} (\pi a_{in}^{\bmod } /V_\pi)$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
%         text(5.1,1.8,'(a)','fontname','Times New Roman','fontsize',12);
    elseif abs(f_cent-1)<delta && p2<1
        p2=1;
        phase_noise_RF2=phase_noise_RF;
        v0_v2=v0_v;
        display(['Generated figure 8(b). Please wait.']);
    elseif abs(f_cent-2)<delta && p3<1
        p3=1;
        phase_noise_RF3=phase_noise_RF;
        v0_v3=v0_v;
        display(['Generated figure 8(c). Please wait.']);
    end

end


sim_end=toc;
figure(1);
subplot(3,2,1)
plot(freq_v.*T,phase_noise_RF1,'-b','LineWidth',1.5);
box off;
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Xlim',[-2.2 2.2],'Xtick',[-2,0,2],'XtickLabel',{'-2';'0';'2'});
set(gca,'Ylim',[-0.1 1.1],'Ytick',[0,1],'YtickLabel',{'0';'1'});
xlabel('$f\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
ylabel('$S_{RF}^{M\tau } (f)\delta f''$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);

subplot(3,2,2)
plot(t_v./T,real(v0_v1).*pi./VpiDC,'-b','LineWidth',1.5);
box off;
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Xlim',[0 5],'Xtick',[0,5],'XtickLabel',{'0';'5'});
set(gca,'Ylim',[-2.5 2.5],'Ytick',[-2,2],'YtickLabel',{'-2';'2'});
xlabel('$T/\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
ylabel('${\mathop{\rm Re}\nolimits} (\pi a_{in}^{\bmod } /V_\pi)$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
text(5.1,1.8,'(a)','fontname','Times New Roman','fontsize',12);

subplot(3,2,3)
plot(freq_v.*T,phase_noise_RF2,'-b','LineWidth',1.5);
box off;
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Xlim',[-2.2 2.2],'Xtick',[-2,0,2],'XtickLabel',{'-2';'0';'2'});
set(gca,'Ylim',[-0.1 1.1],'Ytick',[0,1],'YtickLabel',{'0';'1'});
xlabel('$f\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
ylabel('$S_{RF}^{M\tau } (f)\delta f''$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);

subplot(3,2,4)
plot(t_v./T,real(v0_v2).*pi./VpiDC,'-b','LineWidth',1.5);
box off;
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Xlim',[0 5],'Xtick',[0,5],'XtickLabel',{'0';'5'});
set(gca,'Ylim',[-2.5 2.5],'Ytick',[-2,2],'YtickLabel',{'-2';'2'});
xlabel('$T/\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
ylabel('${\mathop{\rm Re}\nolimits} (\pi a_{in}^{\bmod } /V_\pi)$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
text(5.1,1.8,'(b)','fontname','Times New Roman','fontsize',12);


subplot(3,2,5)
plot(freq_v.*T,phase_noise_RF3,'-b','LineWidth',1.5);
box off;
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Xlim',[-2.2 2.2],'Xtick',[-2,0,2],'XtickLabel',{'-2';'0';'2'});
set(gca,'Ylim',[-0.1 1.1],'Ytick',[0,1],'YtickLabel',{'0';'1'});
xlabel('$f\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
ylabel('$S_{RF}^{M\tau } (f)\delta f''$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);

subplot(3,2,6)
plot(t_v./T,real(v0_v3).*pi./VpiDC,'-b','LineWidth',1.5);
box off;
set(gca,'fontname','Times New Roman','fontsize',12);
set(gca,'Xlim',[0 5],'Xtick',[0,5],'XtickLabel',{'0';'5'});
set(gca,'Ylim',[-2.5 2.5],'Ytick',[-2,2],'YtickLabel',{'-2';'2'});
xlabel('$T/\tau$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
ylabel('${\mathop{\rm Re}\nolimits} (\pi a_{in}^{\bmod } /V_\pi)$','Interpreter','Latex','fontname','Times New Roman','fontsize',12);
text(5.1,1.8,'(c)','fontname','Times New Roman','fontsize',12);
