function [phase_noise,phase_noise_frq,v0_v,t_v,freq_v,phase_noise_RF,phi_v]=SingleOEO_main(N,Nrt,Nmax,T,alpha,RFampCorr,P0,VpiAC,VpiDC,Vb,eta,rhoDC,rho,Ga,attenRF,FilterLoss,Gwidth,CouplerLoss,rho_N)
   

% The simulation of a single OEO was written by Etgar C. Levy in MATLAB software. 
% M. Horowitz and C. Menyuk participated in the design of the algorithms.
% This software is based on the paper:  
% Etgar C. Levy, Moshe Horowitz, and Curtis R. Menyuk, "Modeling optoelectronic oscillators," J. Opt. Soc. Am. B 26, 148-159 (2009).
% The copyright to this software is owned by UMBC and the Technion.

%%%%%%%%%%%
% Parameters %
%%%%%%%%%%%
%rand('state',sum(100*clock));   
DT=T/N; % temporal step-size                       
DF=1/T;  % frequency step-size                      
BW=1/DT;  % simulation bandwidth                     
t=[0:N-1]*DT; % the tau axis (t is equal to tau in the paper)
freq=[-N/2:N/2-1]*DF;   %frequency axis    
w=double(2*pi*freq); %angular frequency
f0=round(10e9*T)/T; %central frequency of the filter, f0=10 GHz              
R=50;   % the impedance at the output of the detector                       
Iph=alpha*P0*rho/2*FilterLoss*CouplerLoss*attenRF;             
Vph=Iph*R*Ga;  % Vph is the photodetector voltage (as in the paper, Eq. 3)              
Gs=-eta*pi*Vph/VpiAC*cos(pi*Vb/VpiDC); % Gs is the small-signal open-loop gain (as in the paper, Eq. 20)  
noise_type='gaussian';
v0=zeros(1,N);  
v1=v0;
v0_m=zeros(N,min(Nrt,Nmax));
v0_m(:,1)=v0;
%%%%%%%%%%%
% OEO RUNNING %
%%%%%%%%%%%

df0=0;
for in=2:Nrt
    %%% RF Coupler - start %%%%
    %v0=CouplerLoss*v0; %v0 in the beginning is the output of the RF ampifier (v0 is the a_out of the amplifier)
    %%% RF Coupler - end %%%%
    
    %%% RF Filter - start %%%
    v2=double(v0);
    f=double(pi*Gwidth*exp(2*pi*i*df0*t).*exp(-pi*Gwidth*t)); %The filter impulse response (Eq. 12 in the paper)
    v02=double(conv([v1 v2],f)./sum(f));
    %takes the result of the convolution from T=tau*(N+1)
    v0=double(v02(N+1:N+N)); %The output from the RF filter, and now v0 is the a_in of the modulator
    v1=double(v2); %now v1 is the old v0
    %%% RF Filter - end %%%
    
    %%%% Optical Modulator - start %%%%
    Pt0=double(alpha*P0/2*(1-eta*sin(pi*Vb/VpiDC)*besselj(0,pi*abs(v0)/VpiAC))); % similar to Eq. 4 in the paper- the DC part
    Pt=-double(alpha*P0*eta*cos(pi*Vb/VpiDC)*besselj(1,pi*abs(v0)/VpiAC).*exp(i*angle(v0))); %similar to Eqs. 5,6 in the paper
    %%%% Optical Modulator - end %%%%
    
    %%%% Photodetector - start %%%%
    i0=double(Pt*rho);
    v0=double(i0*R);
    %%%% Photodetector - end %%%%
    
   
  
    %similar to Eq. 14 (but v0 is here in the time domain)
    %v0=RFampCorr*sqrt(Gamp(abs(v0).^2/2/R)).*(v0+rndnoise(N,R,rho_THERM*N,DF,noise_type)); %Amp saturation and thermal noise 
    v0=double(RFampCorr*(Ga).*(v0+rndnoise(N,R,rho_N*N,DF,noise_type))); %Amp saturation and thermal noise 
    %%%% RF Amplifier and Thermal noise - end %%%%
    
     
    if Nrt-in+1<=Nmax
        v0_m(:,in+Nmax-max(Nrt,Nmax))=double(v0); %save the signal after the steady state (2000 iterations)
    end
    
    %%%% RF attenuation - start %%%% 
    %v0=attenRF*v0;
    %%%% RF attenuation - end   %%%% 
    
    %V0=fftshift(fft(v0))/N;
end
%%%%%%%%%%%%%%%%%%%
% COLLECTING RUNS %
%%%%%%%%%%%%%%%%%%%
v0_v=double(reshape(v0_m(1:N,:),1,(N)*min(Nrt,Nmax)));
N_v=length(v0_v);
T_v=N_v*DT;
t_v=[0:N_v-1]*DT;%the whole T axis 
DF_v=1/T_v;
freq_v=[-N_v/2:N_v/2-1]*DF_v;
phi_v=double(angle(v0_v));
phi_v=unwrap(phi_v);
A_m=double([N_v sum(t_v)
    sum(t_v) sum(t_v.^2)]);
B_v=double([sum(phi_v), sum(t_v.*phi_v)]);
X_v=double(B_v/A_m);
phi_v=double(phi_v-(X_v(1)+X_v(2).*t_v));
V0_v=double(fftshift(fft(v0_v))/N_v);
%G_v=double(Gwidth/2*i./(df0-freq_v+i*Gwidth/2)); %The Fourier transform of f(T) of the filter
%phiG_v=angle(G_v);%phase(G_v);
%%%%%%%%%%%%%%%%%%%%
% OSCILLATOR POWER %
%%%%%%%%%%%%%%%%%%%%
pt=0.5*abs(v0_v).^2/R;                  % Power as a function of time [W]
pavg=trapz(t_v,pt)/t_v(end);            % Average power over cycle time [W]
Pw=0.5*abs(V0_v).^2/R;                  % Power as a function of frequency [W]
Pavg=sum(Pw);                           % Average power over bandwidth [W]
%%%%%%%%%%%%%%%
% PHASE NOISE %
%%%%%%%%%%%%%%%
PHI_v=double(fftshift(fft(phi_v))/N_v); %As in Eq. 16
phase_noise=double(10*log10(abs(PHI_v).^2/DF_v)); %dBc/Hz, As in Eq. 17
%%%%%%%%%%%%%%%%%%%%%%%
% Phase noise from frequency     %
% (assuming no amplitude noise)  %
%%%%%%%%%%%%%%%%%%%%%%%
phase_noise_frq=10*log10(abs(V0_v).^2/R/2/Pavg/DF_v); %As in Eq. 18

phase_noise_RF=abs(V0_v).^2/R/2/Pavg;

